library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity UDP_sub is
Port ( clk, clk3x : in std_logic;
	loc_addr : in  STD_LOGIC_VECTOR (31 downto 0);
	loc_wd : in  STD_LOGIC_VECTOR (7 downto 0);
	loc_we : in  STD_LOGIC;
	loc_re : in  STD_LOGIC;
	loc_ack : out  STD_LOGIC;
	loc_rd : out  STD_LOGIC_VECTOR (7 downto 0);

	t0_in : in  STD_LOGIC;
	t0_out : out  STD_LOGIC;
	t0_on : out  STD_LOGIC;

	tmce : out std_logic;
	tmadd : out std_logic_vector(2 downto 0);
	tmdi : out std_logic_vector(15 downto 0);
	tmdo : in std_logic_vector(15 downto 0)
);
end UDP_sub;

architecture Behavioral of UDP_sub is
component kp_gen50MHz
    Port ( clk : in std_logic;
	caddr : in std_logic_vector(7 downto 0);		--crate address
	t0tm : in std_logic_vector(39 downto 0);		--max time bin
	ptclock : in std_logic_vector(55 downto 0);		--real clock
	nkp : in std_logic;
	rkp : out std_logic);
end component;
	signal ptmce, pptmce : STD_LOGIC;
	signal dgkp, ddgkp, ptsetck, pptsetck, dgclk, ddgclk, pt0_out : STD_LOGIC;
	signal ptclock, ptsetclock : STD_LOGIC_VECTOR(55 downto 0);
	signal gclk_ct: STD_LOGIC_VECTOR(10 downto 0);
	signal adj_ct: STD_LOGIC_VECTOR(2 downto 0);

begin
tmadd <= loc_addr(3 downto 1);
tmce <= ptmce;

U10: kp_gen50MHz port map(clk, X"00", X"0000000000", ptclock, t0_in, pt0_out);
t0_out <= pt0_out;
t0_on <= '1' when ptsetclock>X"01e13380000000" else '0';	--later than 2010

process (CLK) 
begin
if CLK='1' and CLK'event then
	ddgclk <= dgclk;
	if ptsetck='1' then
		ptclock <= ptsetclock;
	elsif dgclk='1' and ddgclk='0' then
		ptclock(10 downto 0) <= (others=>'0');
		ptclock(55 downto 11) <= ptclock(55 downto 11)+1;
	else
		ptclock(10 downto 0) <= ptclock(10 downto 0)+1;
	end if;
	if gclk_ct>="10111110101" or (gclk_ct>="10111110100" and adj_ct="000") then
									--1526(0x5f6) - 1/8 = 32.768kHz @50MHz
		adj_ct <= adj_ct+1;
		gclk_ct <= (others=>'0');
		dgclk <= '1';
	else
		gclk_ct <= gclk_ct+1;
		dgclk <= '0';
	end if;

	pptmce <= ptmce;
	pptsetck <= ptsetck;
end if;
end process;

process (CLK3x) 
begin
if CLK3x='1' and CLK3x'event then
	loc_ack <= loc_we or loc_re;
	if pptmce='1' then	ptmce <= '0';	end if; 
	if pptsetck='1' then	ptsetck <= '0';	end if; 
	if loc_we='1' then
		if loc_addr(31 downto 3)="0000000000000000000000110000" then	--0x180~187
			if loc_addr(0)='0' then
				tmdi(15 downto 8) <= loc_wd;
			else
				tmdi(7 downto 0) <= loc_wd;
				ptmce <= '1';
			end if;
		elsif loc_addr(31 downto 3)="0000000000000000000000110010" then	--0x190~197
			case loc_addr(2 downto 0) is
			when "000"=>	ptsetclock(55 downto 48) <= loc_wd;
			when "001"=>	ptsetclock(47 downto 40) <= loc_wd;
			when "010"=>	ptsetclock(39 downto 32) <= loc_wd;
			when "011"=>	ptsetclock(31 downto 24) <= loc_wd;
			when "100"=>	ptsetclock(23 downto 16) <= loc_wd;
			when "101"=>	ptsetclock(15 downto 8) <= loc_wd;
			when "110"=>	ptsetclock(7 downto 0) <= loc_wd;
								ptsetck <= '1';
			when others=>
			end case;
		end if;
	elsif loc_re='1' then
		if loc_addr(31 downto 3)="0000000000000000000000110000" then	--0x180~187
			if loc_addr(0)='0' then
				loc_rd <= tmdo(15 downto 8);
			else
				loc_rd <= tmdo(7 downto 0);
			end if;
		elsif loc_addr(31 downto 3)="0000000000000000000000110010" then	--0x190~197	--0x198~19f
			case loc_addr(2 downto 0) is
			when "000"=>	loc_rd <= ptclock(55 downto 48);
			when "001"=>	loc_rd <= ptclock(47 downto 40);
			when "010"=>	loc_rd <= ptclock(39 downto 32);
			when "011"=>	loc_rd <= ptclock(31 downto 24);
			when "100"=>	loc_rd <= ptclock(23 downto 16);
			when "101"=>	loc_rd <= ptclock(15 downto 8);
			when "110"=>	loc_rd <= ptclock(7 downto 0);
			when others=>	loc_rd <= (others=>'0');
			end case;
		end if;
	end if;

end if;
end process;

end Behavioral;
