library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
library UNISIM;
use UNISIM.VComponents.all;

entity fifo16k8s is
 port (clk : in std_logic; 
 	srst  : in std_logic; 
  	din  : in std_logic_vector(7 downto 0); 
	wr_en  : in std_logic; 
 	rd_en  : in std_logic; 
  	dout  : out std_logic_vector(7 downto 0); 
	full  : out std_logic; 
 	empty  : out std_logic
); 
end fifo16k8s;

architecture Behavioral of fifo16k8s is
component fifo128_8
  PORT (
    clk : IN STD_LOGIC;
    srst : IN STD_LOGIC;
    din : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
    wr_en : IN STD_LOGIC;
    rd_en : IN STD_LOGIC;
    dout : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
    full : OUT STD_LOGIC;
    empty : OUT STD_LOGIC
  );
END component;
 signal read_a, pread_a : std_logic_vector(13 downto 0); 
 signal write_a : std_logic_vector(13 downto 0); 
 signal mwea : std_logic_vector(3 downto 0); 
 signal pempty, dempty, ddempty, pfull, prd_en, dprd_en, ddprd_en : std_logic; 
 signal pdout : std_logic_vector(7 downto 0); 

begin
GEN1:	for i in 0 to 7 generate
--unita: RAM128X1D port map (DPO=>dout(i),A=>write_a, D=>din(i), DPRA=>pread_a,
-- WCLK=>clk,WE=>wr_en);
unita: RAMB16BWER generic map(DATA_WIDTH_A=>1,DATA_WIDTH_B=>1)
 port map (DOB(0)=>dout(i),DOB(31 downto 1)=>open,ADDRA=>write_a,ADDRB=>pread_a,
 DIA(0)=>din(i),DIA(31 downto 1)=>"0000000000000000000000000000000",
 CLKA=>clk,CLKB=>clk,WEA=>mwea,	WEB=>"0000", RSTA=>'0',RSTB=>'0',
 DIB=>X"00000000", DIPA=>X"0", DIPB=>X"0", REGCEA => '0', REGCEB => '0', ENA=>'1', ENB=>'1');
end generate;
mwea <= "1111" when wr_en='1' else "0000";

--u1: fifo128_8 port map(clk, srst, pdout, ddprd_en, rd_en, dout, pfull, empty);

empty <= '1' when write_a=read_a else '0';
full <= '1' when write_a+1=read_a else '0';
pread_a <= read_a+1 when prd_en='1' and read_a/=write_a else read_a;
prd_en <= '1' when pempty='1' and dempty='1' else '0';

process (clk, wr_en, read_a, write_a) 
begin
if (clk'event and clk = '1') then
	ddprd_en <= dprd_en;
	dprd_en <= prd_en;
	dempty <= pempty;
	if write_a=read_a and pempty='0' and pfull='0' then
		pempty <= '1';
	else
		pempty <= '0';
	end if;

	if srst='1' then
		read_a <= (others=>'0');
		write_a <= (others=>'0');
	else 
 		if wr_en='1' and read_a/=(write_a+1) then 
			write_a <= write_a+1;
 		end if; 
 		if rd_en='1' and read_a/=write_a then
			read_a <= read_a+1;
 		end if; 
 	end if; 
end if; 
 end process;  
end Behavioral;
